@@ -2,6 +2,9 @@
    an extension to C, as they do allocations with a cognizance of C 
    variables.  You can use them to make C read more like a high level
    language.
+
+   Before including this, you must define an __inline__ macro if your
+   compiler doesn't recognize it as a keyword.
 */
 
 #ifndef MALLOCVAR_INCLUDED
@@ -12,6 +15,13 @@
 #include <limits.h>
 #include <stdlib.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+#if 0
+} /* to fake out automatic code indenters */
+#endif
+
 static __inline__ void
 mallocProduct(void **      const resultP, 
               unsigned int const factor1,
@@ -22,6 +32,8 @@ mallocProduct(void **      const resultP,
    return NULL without allocating anything.  Also return NULL if the malloc
    fails.
 
+   If either factor is zero, malloc a single byte.
+
    Note that malloc() actually takes a size_t size argument, so the
    proper test would be whether the size can be represented by size_t,
    not unsigned int.  But there is no reliable indication available to
@@ -29,10 +41,14 @@ mallocProduct(void **      const resultP,
    assume size_t is at least as expressive as unsigned int and that
    nobody really needs to allocate more than 4GB of memory.
 -----------------------------------------------------------------------------*/
-    if (UINT_MAX / factor2 < factor1) 
-        *resultP = NULL; \
-    else 
-        *resultP = malloc(factor1 * factor2); 
+    if (factor1 == 0 || factor2 == 0)
+        *resultP = malloc(1);
+    else {
+        if (UINT_MAX / factor2 < factor1) 
+            *resultP = NULL;
+        else 
+            *resultP = malloc(factor1 * factor2); 
+    }
 }
 
 
@@ -85,5 +101,7 @@ do { \
 #define MALLOCVAR_NOFAIL(varName) \
     do {if ((varName = malloc(sizeof(*varName))) == NULL) abort();} while(0)
 
+#ifdef __cplusplus
+}
+#endif
 #endif
-
